using UnityEngine;
using System.Collections.Generic;
using System.Linq;


namespace JayJayWon
{
    public static class EnumManifest
    {
        private static Dictionary<string, List<int>> enumManifestDict;
        private static Dictionary<string, string> enumValStoreValDict;
        private static Dictionary<string, string> enumValAltDisplayValDict;
        private static Dictionary<string, int> storeValEnumValDict;
        private static Dictionary<string, int> altDisplayValEnumValDict;

        static EnumManifest()
        {
            enumManifestDict = new Dictionary<string, List<int>>();
            enumValStoreValDict = new Dictionary<string, string>();
            enumValAltDisplayValDict = new Dictionary<string, string>();
            storeValEnumValDict = new Dictionary<string, int>();
            altDisplayValEnumValDict = new Dictionary<string, int>();
        }

        static public void AddEnum(string enumName, int enumVal)
        {
            string enumValKey = EnumValKey(enumName, enumVal);
            if (!enumManifestDict.ContainsKey(enumName)) enumManifestDict.Add(enumName, new List<int>());
            if (!enumManifestDict[enumName].Contains(enumVal)) enumManifestDict[enumName].Add(enumVal);
            else SuperController.LogError("UIAssist.EnumManifest.AddEnum 1: Attempt to add existing enumeration value - " + enumValKey);
        }

        static private string EnumValKey(string enumName, int enumVal)
        {
            return enumName + "." + enumVal.ToString();
        }
        static private string EnumLabelKey(string enumName, string enumLabel)
        {
            return enumName + "." + enumLabel;
        }

        static public void AddEnum(string enumName, int enumVal, string storeValLabel)
        {
            AddEnum(enumName, enumVal);
            string enumValKey = EnumValKey(enumName, enumVal);
            string enumStoreLabelKey = EnumLabelKey(enumName, storeValLabel);
            if (!enumValStoreValDict.ContainsKey(enumName))
            {
                enumValStoreValDict.Add(enumValKey, storeValLabel);
                storeValEnumValDict.Add(enumStoreLabelKey,enumVal);
            }
            else SuperController.LogError("UIAssist.EnumManifest.AddEnum 2: Attempt to add existing enumeration value - " + enumValKey);
        }

        static public void AddEnum(string enumName, int enumVal, string storeValLabel, string altLabel)
        {
            AddEnum(enumName, enumVal, storeValLabel);
            string enumValKey = EnumValKey(enumName, enumVal);
            string enumAltLabelKey = EnumLabelKey(enumName, altLabel);
            if (altLabel != null)
            {
                if (!enumValAltDisplayValDict.ContainsKey(enumName))
                {
                    enumValAltDisplayValDict.Add(enumValKey, altLabel);
                    altDisplayValEnumValDict.Add(enumAltLabelKey, enumVal);
                }
                else SuperController.LogError("UIAssist.EnumManifest.AddEnum 3: Attempt to add existing enumeration value - " + enumValKey);
            }

        }

        static public bool Exists(string enumName, int enumVal)
        {
            if (!enumManifestDict.ContainsKey(enumName)) return false;
            if (!enumManifestDict[enumName].Contains(enumVal)) return false;
            return (true);
        }

        static public string GetDisplayVal(string enumName, int enumVal)
        {
            string enumValKey = EnumValKey(enumName, enumVal);
            if (Exists(enumName, enumVal))
            {
                if (enumValAltDisplayValDict.ContainsKey(enumValKey)) return enumValAltDisplayValDict[enumValKey];
                if (enumValStoreValDict.ContainsKey(enumValKey)) return enumValStoreValDict[enumValKey];
            }
            else SuperController.LogError("UIAssist.EnumManifest.GetLabel: Attempt to retrieve display val for non-existing enumeration value - " + enumValKey);
            return ("");
        }
        static public string GetStoreVal(string enumName, int enumVal)
        {
            string enumValKey = EnumValKey(enumName, enumVal);
            if (Exists(enumName, enumVal))
            {
                if (enumValStoreValDict.ContainsKey(enumValKey)) return enumValStoreValDict[enumValKey];
            }
            else SuperController.LogError("UIAssist.EnumManifest.GetStoreVal: Attempt to retrieve Store Value for non-existing enumeration value - " + enumValKey);
            return ("");
        }


        static public int GetEnumValFromStoreVal(string enumName, string storeValLabel)
        {
            string enumStoreLabelKey = EnumLabelKey(enumName, storeValLabel);
            if (storeValEnumValDict.ContainsKey(enumStoreLabelKey)) return storeValEnumValDict[enumStoreLabelKey];
            else SuperController.LogError("UIAssist.EnumManifest.GetEnumValFromStoreVal: Attempt to retrieve enumeration Value for non-existing enumeration Label - " + enumStoreLabelKey);
            return (-1);
        }

        static public int GetEnumValFromDisplayVal(string enumName, string displayVal)
        {
            string enumStoreLabelKey = EnumLabelKey(enumName, displayVal);
            if (altDisplayValEnumValDict.ContainsKey(enumStoreLabelKey)) return altDisplayValEnumValDict[enumStoreLabelKey];
            else if (storeValEnumValDict.ContainsKey(enumStoreLabelKey)) return storeValEnumValDict[enumStoreLabelKey];
            else SuperController.LogError("UIAssist.EnumManifest.GetEnumValFromDisplayVal: Attempt to retrieve enumeration Value for non-existing enumeration Display Val - " + enumStoreLabelKey);
            return (-1);
        }

        public static List<string> GetDisplayValList(string enumName)
        {
            List<string> displayVals = new List<string>();
            if (enumManifestDict.ContainsKey(enumName))
            {
                foreach (KeyValuePair<string, string> kvp in enumValStoreValDict)
                {
                    if (kvp.Key.StartsWith(enumName + "."))
                    {
                        if (enumValAltDisplayValDict.ContainsKey(kvp.Key)) displayVals.Add(enumValAltDisplayValDict[kvp.Key]);
                        else displayVals.Add(enumValStoreValDict[kvp.Key]);
                    }
                }
            }

            return (displayVals);
        }

        public static List<int> GetEnumsList(string enumName)
        {
            if (enumName == null) return null;
            List<int> enums = new List<int>();
            if (enumManifestDict.ContainsKey(enumName))
            {
                foreach (int enumVal in enumManifestDict[enumName])
                {
                    
                    enums.Add(enumVal);
                }
            }

            return (enums);
        }
        public static SortedDictionary<int,string> GetEnumsDisplayValDict(string enumName)
        {
            if (enumName == null) return null;
            SortedDictionary<int,string> enumsDict = new SortedDictionary<int, string>();
            if (enumManifestDict.ContainsKey(enumName))
            {
                foreach (int enumVal in enumManifestDict[enumName])
                {
                    enumsDict.Add(enumVal, GetDisplayVal(enumName, enumVal));
                }
            }

            return (enumsDict);
        }

        public static Dictionary<int, string> GetEnumsStoreValDict(string enumName)
        {
            if (enumName == null) return null;
            Dictionary<int, string> enumsDict = new Dictionary<int, string>();
            if (enumManifestDict.ContainsKey(enumName))
            {
                foreach (int enumVal in enumManifestDict[enumName])
                {
                    enumsDict.Add(enumVal, GetStoreVal(enumName, enumVal));
                }
            }

            return (enumsDict);
        }

        public static Dictionary<string, int> GetStoreValEnumsDict(string enumName)
        {
            if (enumName == null) return null;
            Dictionary<string, int> enumsDict = new Dictionary<string, int>();
            if (enumManifestDict.ContainsKey(enumName))
            {
                foreach (int enumVal in enumManifestDict[enumName])
                {
                    enumsDict.Add(GetStoreVal(enumName, enumVal), enumVal);
                }
            }

            return (enumsDict);
        }
        public static Dictionary<string, int> GetDisplayValEnumsDict(string enumName)
        {
            if (enumName == null) return null;
            Dictionary<string, int> enumsDict = new Dictionary<string, int>();
            if (enumManifestDict.ContainsKey(enumName))
            {
                foreach (int enumVal in enumManifestDict[enumName])
                {
                    enumsDict.Add(GetDisplayVal(enumName, enumVal), enumVal);
                }
            }

            return (enumsDict);
        }

    }

    public static class LeftRight
    {
        public const int left = 1;
        public const int right = 2;
        public const int both = 3;
        public const int neither = 4;

        public static string enumManifestName
        {
            get { return "LeftRight"; }
        }
        static LeftRight()
        {
            EnumManifest.AddEnum(enumManifestName, left);
            EnumManifest.AddEnum(enumManifestName, right);
            EnumManifest.AddEnum(enumManifestName, both);
            EnumManifest.AddEnum(enumManifestName, neither);
        }
    }

    public static class UIHelpExpandMode
    {
        public const int minimized = 1;
        public const int half = 2;
        public const int full = 3;

        public static string enumManifestName
        {
            get { return "UIHelpExpandMode"; }
        }
        static UIHelpExpandMode()
        {
            EnumManifest.AddEnum(enumManifestName, minimized, "minimized");
            EnumManifest.AddEnum(enumManifestName, half, "half");
            EnumManifest.AddEnum(enumManifestName, full, "full");

        }
    }


    public static class CGTNameFilterTypes
    {
        public const int startsWith = 1;
        public const int endsWith = 2;
        public const int contains = 3;
        public const int none = 4;

        public static string enumManifestName
        {
            get { return "CGTNameFilterTypes"; }
        }
        static CGTNameFilterTypes()
        {
            EnumManifest.AddEnum(enumManifestName, startsWith, "Starts with...");
            EnumManifest.AddEnum(enumManifestName, endsWith, "Ends with...");
            EnumManifest.AddEnum(enumManifestName, contains, "Contains...");
            EnumManifest.AddEnum(enumManifestName, none, "None");
        }
    }
    public static class GameControlUIModes
    {
        public const int desktopHUD = 1;
        public const int vrHUD = 2;
        public const int vrStatic = 3;
        public const int vrWrist = 4;
    }

    public static class NearestUIButtonType
    {
        public const int none = 1;
        public const int grid = 2;
        public const int nonGrid = 3;
    }

    public static class GameControlDisplayModes
    {
        public const int standard = 1;
        public const int ace = 2;
        public const int edit = 3;
        public const int atomSelect = 4;
    }

    public static class CGTGenderFilterTypes
    {
        public const int femaleOnly = 1;
        public const int maleOnly = 2;
        public const int any = 3;

        public static string enumManifestName
        {
            get { return "CGTGenderFilterTypes"; }
        }
        static CGTGenderFilterTypes()
        {
            EnumManifest.AddEnum(enumManifestName, femaleOnly, "Female only");
            EnumManifest.AddEnum(enumManifestName, maleOnly, "Male only");
            EnumManifest.AddEnum(enumManifestName, any, "Any");
        }
    }


    public static class VRHandControl
    {
        public const int leftHand = 1;
        public const int rightHand = 2;
        public const int disabled = 3;

        public static string enumManifestName
        {
            get { return "VRHandControl"; }
        }
        static VRHandControl()
        {
            EnumManifest.AddEnum(enumManifestName, leftHand, "Left Hand");
            EnumManifest.AddEnum(enumManifestName, rightHand,"Right Hand");
            EnumManifest.AddEnum(enumManifestName, disabled,"Disabled");
        }
    }

    public static class VRPinMode
    {
        public const int staticMode =1;
        public const int hudLMode = 2;
        public const int hudRMode = 3;
        public const int hudCustomMode = 4;
        public const int disabled = 5;

        public static string enumManifestName
        {
            get { return "VRPinMode"; }
        }

        static VRPinMode()
        {
            EnumManifest.AddEnum(enumManifestName, staticMode, "Static");
            EnumManifest.AddEnum(enumManifestName, hudLMode, "HUD (Left)");
            EnumManifest.AddEnum(enumManifestName, hudRMode, "HUD (Right)");
            EnumManifest.AddEnum(enumManifestName, hudCustomMode, "HUD (Custom)");
            EnumManifest.AddEnum(enumManifestName, disabled, "Disabled");
        }
    }

    public static class VRGazeUIControlMode
    {
        public const int wristUIOnly = 1;
        public const int wristAndPinnedUI = 2;
        public const int disabled = 3;

        public static string enumManifestName
        {
            get { return "VRGazeUIControlMode"; }
        }

        static VRGazeUIControlMode()
        {
            EnumManifest.AddEnum(enumManifestName, wristUIOnly, "Wrist UI Only");
            EnumManifest.AddEnum(enumManifestName, wristAndPinnedUI, "Wrist & Pinned UI");
            EnumManifest.AddEnum(enumManifestName, disabled, "Disabled");
        }
    }

    public static class ButtonUIMode
    {
        public const int noMode = 0;
        public const int repositionButtonMode = 1;
        public const int repositionButtonOpMode = 2;
        public const int copyButtonOpMode = 3;
        public const int deleteButtonOpMode = 4;
        public const int copyButtonMode = 5;
        public const int clearButtonMode = 6;
    }

    public static class GridUIMode
    {
        public const int noMode = 0;
        public const int repositionGridMode = 1;
        public const int deleteGridMode = 2;
    }

    public static class TestSpawnMode
    {
        public const int teleportAtomMode = 1;
        public const int spawnAtomMode = 2;
    }

    public static class ButtonState
    {
        public const int inactive = 1;
        public const int active = 2;

        public static string enumManifestName
        {
            get { return "ButtonState"; }
        }
        static ButtonState()
        {
            EnumManifest.AddEnum(enumManifestName, inactive);
            EnumManifest.AddEnum(enumManifestName, active);
        }
    }

    public static class TopUITabs
    {
        public const int buttonSetup = 1;
        public const int loadConfig = 2;
        public const int saveConfig = 3;
        public const int tools = 4;
        public const int settings = 5;

    }


    public static class ClothingActionMode
    {
        public const int undress = 1;
        public const int resetSim = 2;
        public const int remove = 3;
        public const int merge = 4;

    }

    public static class LastViewedTargetType
    {
        public const int lastViewedPerson = 1;
        public const int lastViewedFemale = 2;
        public const int lastViewedMale = 3;
        public const int lastViewedAtom = 4;
        public const int lastViewedNonPerson = 5;

        public static string enumManifestName
        {
            get { return "LastViewedTarget"; }
        }
        static LastViewedTargetType()
        {
            EnumManifest.AddEnum(enumManifestName, lastViewedPerson,"Last viewed Person");
            EnumManifest.AddEnum(enumManifestName, lastViewedFemale, "Last viewed Female");
            EnumManifest.AddEnum(enumManifestName, lastViewedMale, "Last viewed Male");
            EnumManifest.AddEnum(enumManifestName, lastViewedAtom, "Last viewed Atom");
            EnumManifest.AddEnum(enumManifestName, lastViewedNonPerson, "Last viewed Non-Person Atom");
        }

        public static List<string> storeValList
        {
            get
            {
                return (EnumManifest.GetEnumsStoreValDict(enumManifestName).Values.ToList());
            }
        }
    }

    public static class LastSelectedTargetType
    {
        public const int lastSelectedPerson = 1;
        public const int lastSelectedFemale = 2;
        public const int lastSelectedMale = 3;
        public const int lastSelectedAtom = 4;
        public const int lastSelectedNonPerson = 5;

        public static string enumManifestName
        {
            get { return "LastSelectedTarget"; }
        }
        static LastSelectedTargetType()
        {
            EnumManifest.AddEnum(enumManifestName, lastSelectedPerson, "Last selected Person");
            EnumManifest.AddEnum(enumManifestName, lastSelectedFemale, "Last selected Female");
            EnumManifest.AddEnum(enumManifestName, lastSelectedMale, "Last selected Male");
            EnumManifest.AddEnum(enumManifestName, lastSelectedAtom, "Last selected Atom");
            EnumManifest.AddEnum(enumManifestName, lastSelectedNonPerson, "Last selected Non-Person Atom");
        }
        public static List<string> storeValList
        {
            get
            {
                return (EnumManifest.GetEnumsStoreValDict(enumManifestName).Values.ToList());
            }
        }
    }
    public static class AllAtomsTargetType
    {
        public const int allPersonAtoms = 1;
        public const int allFemaleAtoms = 2;
        public const int allMaleAtoms = 3;
        public const int allAtoms = 4;
        public const int allNonPersonAtoms = 5;

        public static string enumManifestName
        {
            get { return "AllAtomsTarget"; }
        }
        static AllAtomsTargetType()
        {
            EnumManifest.AddEnum(enumManifestName, allPersonAtoms, "All People");
            EnumManifest.AddEnum(enumManifestName, allFemaleAtoms, "All Females");
            EnumManifest.AddEnum(enumManifestName, allMaleAtoms, "All Males");
            EnumManifest.AddEnum(enumManifestName, allAtoms, "All Atoms");
            EnumManifest.AddEnum(enumManifestName, allNonPersonAtoms, "All Non-Person Atoms");
        }
        public static List<string> storeValList
        {
            get
            {
                return (EnumManifest.GetEnumsStoreValDict(enumManifestName).Values.ToList());
            }
        }
    }

    public static class UserChosenTargetType
    {
        public const int personAtoms = 11;
        public const int femaleAtoms = 12;
        public const int maleAtoms = 13;
        public const int anyAtoms = 14;
        public const int nonPersonAtoms = 15;

        public static string enumManifestName
        {
            get { return "UserChosenTarget"; }
        }
        static UserChosenTargetType()
        {
            EnumManifest.AddEnum(enumManifestName, personAtoms, "Person Atoms");
            EnumManifest.AddEnum(enumManifestName, femaleAtoms, "Female Atoms");
            EnumManifest.AddEnum(enumManifestName, maleAtoms, "Male Atoms");
            EnumManifest.AddEnum(enumManifestName, anyAtoms, "Any Atoms");
            EnumManifest.AddEnum(enumManifestName, nonPersonAtoms, "Non Person Atoms");
        }
        public static List<string> storeValList
        {
            get
            {
                return (EnumManifest.GetEnumsStoreValDict(enumManifestName).Values.ToList());
            }
        }
    }

    public static class PluginVariableType
    {
        public const int vBool = 1;
        public const int vFloat = 2;
        public const int vString = 3;
        public const int vStringChooser = 4;
        public const int vURL = 5;
        public const int vColor = 6;
        public const int vNone = 7;
        public const int vAll = 8;
        public const int vAction = 9;

        public static string enumManifestName
        {
            get { return "PluginVariableType"; }
        }
        static PluginVariableType()
        {
            EnumManifest.AddEnum(enumManifestName, vBool, "BOOL");
            EnumManifest.AddEnum(enumManifestName, vFloat, "FLOAT");
            EnumManifest.AddEnum(enumManifestName, vString, "STRING");
            EnumManifest.AddEnum(enumManifestName, vStringChooser, "STRINGCHOOSER");
            EnumManifest.AddEnum(enumManifestName, vURL, "URL");
            EnumManifest.AddEnum(enumManifestName, vColor, "COLOR");
            EnumManifest.AddEnum(enumManifestName, vNone, "NONE");
            EnumManifest.AddEnum(enumManifestName, vAll, "ALL");
            EnumManifest.AddEnum(enumManifestName, vAction, "ACTION");
        }
        public static List<string> storeValList
        {
            get
            {
                return (EnumManifest.GetEnumsStoreValDict(enumManifestName).Values.ToList());
            }
        }
    }

    public static class NoneAvailable
    {
        public const int noneAvailable = 1;

        public static string enumManifestName
        {
            get { return "NoneAvailable"; }
        }
        static NoneAvailable()
        {
            EnumManifest.AddEnum(enumManifestName, noneAvailable, "<None Available>");

        }

    }



    public static class FileReferenceCategory
    {
        public const int presetCat = 1;
        public const int legacyPresetCat = 2;
        public const int sceneCat = 3;
        public const int uiapCat = 4;
        public const int addonPackagesFolderCat = 5;
        public const int pluginCat = 6;

        public static string GetFileTypeFilter(int fileRefCategory)
        {
            if (fileRefCategory == presetCat) return "vap";
            if (fileRefCategory == legacyPresetCat) return "json";
            if (fileRefCategory == sceneCat) return "json";
            if (fileRefCategory == uiapCat) return "uiap";
            if (fileRefCategory == pluginCat) return "cs|cslist|dll";
            return "";
        }

    }

    public static class ButtonComponentTypes
    {
        public const int targetComponent = 1;
        public const int pluginSettingComponent = 2;
        public const int pluginsLoadComponent = 3;
        public const int appearancePresetComponent = 4;
        public const int clothingComponent = 5;
        public const int vamPlayEditModeComponent = 6;
        public const int skinPresetDecalComponent = 7;
        public const int spawnAtomComponent = 8;
        public const int relativePositionComponent = 9;
        public const int moveAtomComponent = 10;
        public const int motionCaptureComponent = 11;
        public const int worldScaleComponent = 12;
        public const int showVRHandsComponent = 13;
        public const int switchUIAGridComponent = 14;
        public const int decalMakerComponent = 15;
        public const int presetLockComponent = 16;
        public const int hairColorComponent = 17;
        public const int buttonSkinComponent = 18;
        public const int nodeControlOnComponent = 19;
        public const int nodeControlOffComponent = 20;
        public const int nodePhysicsOnComponent = 21;
        public const int nodePhysicsOffComponent = 22;
        public const int nodeSelectionComponent = 23;

        public const int button = 100;
        public const int operation = 101;
        public const int grid = 102;
        public const int fileRefs = 103;
    }


    public static class FileReferenceTypes
    {
        public const int appearancePreset = 1;
        public const int clothingPreset = 2;
        public const int posePreset = 3;
        public const int hairPreset = 4;
        public const int pluginsPreset = 5;
        public const int morphPreset = 6;
        public const int skinPreset = 7;
        public const int glutePreset = 8;
        public const int breastPreset = 9;
        public const int generalPreset = 10;
        public const int legacyPreset = 11;
        public const int legacyPosePreset = 12;
        public const int legacyLookPreset = 13;
        public const int scene = 14;
        public const int subScene = 15;
        public const int uiap = 16;
        public const int addonPackagesFolder = 17;
        public const int plugin = 18;
        public const int sessionPluginPreset = 19;
        public const int scenePluginPreset = 20;
        public const int mocap = 21;

        private static Dictionary<int, int> fileTypeToCategoryDict = new Dictionary<int, int>();

        private static Dictionary<int, string> fileTypeToDefaultFolderDict = new Dictionary<int, string>();
        private static Dictionary<int, string> fileRefTypeDescription = new Dictionary<int, string>();

        public static string enumManifestName
        {
            get { return "FileReferenceTypes"; }
        }
        static FileReferenceTypes()
        {
            InitEnum(enumManifestName, appearancePreset, "appearancePreset",FileReferenceCategory.presetCat, "Custom\\Atom\\Person\\Appearance","Appearance Preset");
            InitEnum(enumManifestName, clothingPreset, "clothingPreset", FileReferenceCategory.presetCat, "Custom\\Atom\\Person\\Clothing","Clothing Preset");
            InitEnum(enumManifestName, posePreset, "posePreset", FileReferenceCategory.presetCat, "Custom\\Atom\\Person\\Pose", "Pose Preset");
            InitEnum(enumManifestName, hairPreset, "hairPreset", FileReferenceCategory.presetCat, "Custom\\Atom\\Person\\Hair", "Hair Preset");

            InitEnum(enumManifestName, pluginsPreset, "pluginsPreset", FileReferenceCategory.presetCat, "Custom\\Atom\\Person\\Plugins", "Plugins Preset");
            InitEnum(enumManifestName, morphPreset, "morphPreset", FileReferenceCategory.presetCat, "Custom\\Atom\\Person\\Morphs", "Morph Preset");
            InitEnum(enumManifestName, skinPreset, "skinPreset", FileReferenceCategory.presetCat, "Custom\\Atom\\Person\\Skin", "Skin Preset");
            InitEnum(enumManifestName, glutePreset, "glutePreset", FileReferenceCategory.presetCat, "Custom\\Atom\\Person\\GlutePhysics", "Glute Physics Preset");
            InitEnum(enumManifestName, breastPreset, "breastPreset", FileReferenceCategory.presetCat, "Custom\\Atom\\Person\\BreastPhysics", "Breast Physics Preset");
            InitEnum(enumManifestName, generalPreset, "generalPreset", FileReferenceCategory.presetCat, "Custom\\Atom", "General Preset");
            InitEnum(enumManifestName, legacyPreset, "legacyPreset", FileReferenceCategory.legacyPresetCat, "Saves\\Person\\full", "Legacy Preset");
            InitEnum(enumManifestName, legacyPosePreset, "legacyPosePreset", FileReferenceCategory.legacyPresetCat, "Saves\\Person\\pose","Legacy Pose");
            InitEnum(enumManifestName, legacyLookPreset, "legacyLookPreset", FileReferenceCategory.legacyPresetCat, "Saves\\Person\\appearance","Legacy Look");
            InitEnum(enumManifestName, scene, "scene", FileReferenceCategory.sceneCat, "Saves\\scene","Scene");
            InitEnum(enumManifestName, subScene, "subScene", FileReferenceCategory.sceneCat, "Custom\\SubScene", "SubScene");
            InitEnum(enumManifestName, uiap, "uiap", FileReferenceCategory.uiapCat, UIAConsts._PluginDataSubfolderName, "UIAssist Profile");
            InitEnum(enumManifestName, addonPackagesFolder, "addonPackagesFolder", FileReferenceCategory.addonPackagesFolderCat,"AddonPackages","AddonPackages Folder");
            InitEnum(enumManifestName, plugin, "plugin", FileReferenceCategory.pluginCat,"Custom\\Scripts","Plugin Script");
            InitEnum(enumManifestName, scenePluginPreset, "scenePluginPreset", FileReferenceCategory.presetCat, "Custom\\PluginPresets","Scene Plugin Preset");
            InitEnum(enumManifestName, sessionPluginPreset, "sessionPluginPreset", FileReferenceCategory.presetCat, "Custom\\PluginPresets","Session Plugin Preset");
            InitEnum(enumManifestName, mocap, "mocap", FileReferenceCategory.sceneCat, "Saves\\mocap", "EverLaster Mocap");
        }

        static void InitEnum(string enumName, int enumVal, string storeValLabel, int fileRefCat, string defaultFolder, string description)
        {
            EnumManifest.AddEnum(enumName, enumVal, storeValLabel);
            fileTypeToCategoryDict.Add(enumVal, fileRefCat);
            fileTypeToDefaultFolderDict.Add(enumVal, defaultFolder);
            fileRefTypeDescription.Add(enumVal, description);
        }
        public static string GetFileRefDescription(int fileRefType)
        {
            return fileRefTypeDescription[fileRefType];
        }
        public static int GetFileRefCategory(int fileRefType)
        {
            return fileTypeToCategoryDict[fileRefType];
        }

        public static string GetDefaultFolder(int fileRefType)
        {
            return fileTypeToDefaultFolderDict[fileRefType];
        }

        public static string GetPrefixRemoval(int fileRefType)
        {
            if (GetFileRefCategory(fileRefType) == FileReferenceCategory.presetCat) return "Preset_";
            if (fileRefType == scenePluginPreset || fileRefType == sessionPluginPreset) return "Plugins_";

            return "";
        }
    }

    public static class RelativePositionMode
    {
        public const int lastViewedPerson = 1;
        public const int lastViewedFemale = 2;
        public const int lastViewedMale = 3;

        public const int sceneOrigin = 5;
        public const int vrHead = 6;
        public const int vrLeftHand = 7;
        public const int vrRightHand = 8;

        public static string enumManifestName
        {
            get { return "RelativePositionMode"; }
        }
        static RelativePositionMode()
        {
            InitEnum(enumManifestName, lastViewedPerson, "Last viewed Person");
            InitEnum(enumManifestName, lastViewedFemale, "Last viewed Female");
            InitEnum(enumManifestName, lastViewedMale, "Last viewed Male");

            InitEnum(enumManifestName, sceneOrigin, "Scene Origin");
            InitEnum(enumManifestName, vrHead, "VR Head");
            InitEnum(enumManifestName, vrLeftHand, "VR Left Hand");
            InitEnum(enumManifestName, vrRightHand, "VR Right Hand");
        }

        static void InitEnum(string enumName, int enumVal, string storeValLabel)
        {
            EnumManifest.AddEnum(enumName, enumVal, storeValLabel);
        }
    }

    public static class AtomCategories
    {
        public const int people = 1;
        public const int misc = 2;
        public const int animation = 3;
        public const int force = 4;
        public const int light = 5;
        public const int toys = 6;
        public const int sound = 7;
        public const int triggers = 8;

        public static string enumManifestName
        {
            get { return "AtomCategories"; }
        }
        static AtomCategories()
        {
            InitEnum(enumManifestName, people, "People");
            InitEnum(enumManifestName, misc, "Misc");
            InitEnum(enumManifestName, animation, "Animation");
            InitEnum(enumManifestName, force, "Force");
            InitEnum(enumManifestName, light, "Light");
            InitEnum(enumManifestName, toys, "Toys");
            InitEnum(enumManifestName, sound, "Sound");
            InitEnum(enumManifestName, triggers, "Triggers");

        }

        static void InitEnum(string enumName, int enumVal, string storeValLabel)
        {
            EnumManifest.AddEnum(enumName, enumVal, storeValLabel);
        }

        public static List<string> GetAtomCategories()
        {
            return EnumManifest.GetDisplayValList(enumManifestName);
        }
    }

    public static class AtomTypes
    {
        public const int person = 1;

        public const int cua = 2;
        public const int clothGraphSphere = 3;
        public const int empty = 4;
        public const int imagePanel = 5;
        public const int imagePanelEmissive = 6;
        public const int imagePanelTransparent = 7;
        public const int imagePanelTransparentEmissive = 8;
        public const int simpleSign = 9;
        public const int subScene = 10;
        public const int uiText = 11;
        public const int webBrowser = 12;
        public const int webPanel = 13;
        public const int webPanelEmissive = 14;

        public const int animationPattern = 15;

        public const int cycleForce = 16;
        public const int grabPoint = 17;
        public const int rhythmForce = 18;
        public const int syncForce = 19;

        public const int invisibleLight = 20;
        public const int torch = 21;

        public const int dildo = 22;
        public const int toyAH = 23;
        public const int toyBP = 24;
        public const int paddle = 25;

        public const int aptSpeaker = 26;
        public const int audioSource = 27;
        public const int rhythmAudioSource = 28;

        public const int button = 29;
        public const int collisionTrigger = 30;
        public const int lookAtTrigger = 31;
        public const int uiButton = 32;
        public const int uiSlider = 33;
        public const int uiToggle = 34;
        public const int variableTrigger = 35;

        private static Dictionary<int, List<int>> atomCategoryToTypes = new Dictionary<int, List<int>>();

        public static string enumManifestName
        {
            get { return "AtomTypes"; }
        }
        static AtomTypes()
        {
            InitEnum(enumManifestName, person, "Person",AtomCategories.people);

            InitEnum(enumManifestName, cua, "CustomUnityAsset", AtomCategories.misc);
            InitEnum(enumManifestName, clothGraphSphere, "ClothGrabSphere", AtomCategories.misc);
            InitEnum(enumManifestName, empty, "Empty", AtomCategories.misc);
            InitEnum(enumManifestName, imagePanel, "ImagePanel", AtomCategories.misc);
            InitEnum(enumManifestName, imagePanelEmissive, "ImagePanelEmissive", AtomCategories.misc);
            InitEnum(enumManifestName, imagePanelTransparent, "ImagePanelTransparent", AtomCategories.misc);
            InitEnum(enumManifestName, imagePanelTransparentEmissive, "ImagePanelTransparentEmissive", AtomCategories.misc);
            InitEnum(enumManifestName, simpleSign, "SimpleSign", AtomCategories.misc);
            InitEnum(enumManifestName, subScene, "SubScene", AtomCategories.misc);
            InitEnum(enumManifestName, uiText, "UIText", AtomCategories.misc);
            InitEnum(enumManifestName, webBrowser, "WebBrowser", AtomCategories.misc);
            InitEnum(enumManifestName, webPanel, "WebPanel", AtomCategories.misc);
            InitEnum(enumManifestName, webPanelEmissive, "WebPanelEmissive", AtomCategories.misc);

            InitEnum(enumManifestName, animationPattern, "AnimationPattern", AtomCategories.animation);

            InitEnum(enumManifestName, cycleForce, "CycleForce", AtomCategories.force);
            InitEnum(enumManifestName, grabPoint, "GrabPoint", AtomCategories.force);
            InitEnum(enumManifestName, rhythmForce, "RhythmForce", AtomCategories.force);
            InitEnum(enumManifestName, syncForce, "SyncForce", AtomCategories.force);

            InitEnum(enumManifestName, invisibleLight, "InvisibleLight", AtomCategories.light);
            InitEnum(enumManifestName, torch, "Torch", AtomCategories.light);

            InitEnum(enumManifestName, dildo, "Dildo", AtomCategories.toys);
            InitEnum(enumManifestName, toyAH, "ToyAH", AtomCategories.toys);
            InitEnum(enumManifestName, toyBP, "ToyBP", AtomCategories.toys);
            InitEnum(enumManifestName, paddle, "Paddle", AtomCategories.toys);

            InitEnum(enumManifestName, aptSpeaker, "AptSpeaker", AtomCategories.sound);
            InitEnum(enumManifestName, audioSource, "AudioSource", AtomCategories.sound);
            InitEnum(enumManifestName, rhythmAudioSource, "RhythmAudioSource", AtomCategories.sound);

            InitEnum(enumManifestName, button, "Button", AtomCategories.triggers);
            InitEnum(enumManifestName, collisionTrigger, "CollisionTrigger", AtomCategories.triggers);
            InitEnum(enumManifestName, lookAtTrigger, "LookAtTrigger", AtomCategories.triggers);
            InitEnum(enumManifestName, uiButton, "UIButton", AtomCategories.triggers);
            InitEnum(enumManifestName, uiSlider, "UISlider", AtomCategories.triggers);
            InitEnum(enumManifestName, uiToggle, "UIToggle", AtomCategories.triggers);
            InitEnum(enumManifestName, variableTrigger, "VariableTrigger", AtomCategories.triggers);
        }

        static void InitEnum(string enumName, int enumVal, string storeValLabel, int atomCategory)
        {
            EnumManifest.AddEnum(enumName, enumVal, storeValLabel);
            if (!atomCategoryToTypes.ContainsKey(atomCategory)) atomCategoryToTypes.Add(atomCategory, new List<int>());
            atomCategoryToTypes[atomCategory].Add(enumVal) ;
        }

        public static List<int> GetAtomTypes(int atomCategory =-1)
        {
            if (atomCategory == -1) return EnumManifest.GetEnumsList(enumManifestName);
            else return atomCategoryToTypes[atomCategory];
        }

        public static List<int> GetAtomTypesExcluding(int atomCategoryToExclude)
        {
            List<int> atomTypes = new List<int>();
            foreach (KeyValuePair<int,List<int>> kvp in atomCategoryToTypes)
            {
                if (kvp.Key != atomCategoryToExclude) atomTypes.AddRange(kvp.Value);
            }
            return atomTypes;
        }
    }

    public static class ButtonSize
    {
        public const int micro = 1;
        public const int mini = 2;
        public const int small = 3;
        public const int medium = 4;
        public const int large = 5;
        public static string enumManifestName
        {
            get { return "ButtonSize"; }
        }

        public static class Micro
        {
            public const float width = 30f;
            public const float height = width;
            public static Vector2 dimensions
            {
                get
                {
                    return new Vector2(width, height);
                }
            }
        }

        public static class Mini
        {
            public const float width = 40f;
            public const float height = width;
            public static Vector2 dimensions
            {
                get
                {
                    return new Vector2(width, height);
                }
            }
        }

        public static class Small
        {
            public const float width = 60f;
            public const float height = width;
            public static Vector2 dimensions
            {
                get
                {
                    return new Vector2(width, height);
                }
            }
        }

        public static class Medium
        {
            public const float width = 90f;
            public const float height = width;
            public static Vector2 dimensions
            {
                get
                {
                    return new Vector2(width, height);
                }
            }
        }

        public static class Large
        {
            public const float width = 120f;
            public const float height = width;
            public static Vector2 dimensions
            {
                get
                {
                    return new Vector2(width, height);
                }
            }
        }
        static ButtonSize()
        {
            EnumManifest.AddEnum(enumManifestName, micro, "Micro");
            EnumManifest.AddEnum(enumManifestName, mini, "Mini");
            EnumManifest.AddEnum(enumManifestName, small, "Small");
            EnumManifest.AddEnum(enumManifestName, medium, "Medium");
            EnumManifest.AddEnum(enumManifestName, large, "Large");
        }

        public static string GetLabel(int buttonSize)
        {
            return EnumManifest.GetDisplayVal(enumManifestName, buttonSize);
        }

        public static int GetEnumVal(string buttonSizeLabel)
        {
            return EnumManifest.GetEnumValFromDisplayVal(enumManifestName, buttonSizeLabel);
        }

        public static List<string> displayValsList
        {
            get
            {
                return (EnumManifest.GetDisplayValList(enumManifestName));
            }
        }

        public static void TouchMe() { }
    }

    public static class UIAButtonCategory
    {
        public const int atomControl = 1;
        public const int nodeControl = 2;
        public const int appearance = 3;
        public const int clothing = 4;
        public const int presets = 5;
        public const int legacyPresets = 6;
        public const int pluginSettings = 7;
        public const int pluginsLoading = 8;
        public const int uiAssist = 9;
        public const int sceneAnim = 10;
        public const int vamUserSettings = 11;
        public const int sceneLoading = 12;
        public const int vrSettings = 13;
        public const int misc = 14;
        public static string enumManifestName
        {
            get { return "UIAButtonCategory"; }
        }

        static UIAButtonCategory()
        {
            EnumManifest.AddEnum(enumManifestName, atomControl, "Atom Control");
            EnumManifest.AddEnum(enumManifestName, nodeControl, "Node Control");
            EnumManifest.AddEnum(enumManifestName, appearance, "Appearance");
            EnumManifest.AddEnum(enumManifestName, clothing, "Clothing");
            EnumManifest.AddEnum(enumManifestName, presets, "Presets");
            EnumManifest.AddEnum(enumManifestName, legacyPresets, "Legacy Presets");
            EnumManifest.AddEnum(enumManifestName, pluginSettings, "Plugin Settings");
            EnumManifest.AddEnum(enumManifestName, pluginsLoading, "Plugin Loading");
            EnumManifest.AddEnum(enumManifestName, uiAssist, "UI Assist");
            EnumManifest.AddEnum(enumManifestName, sceneAnim, "Scene Animation");
            EnumManifest.AddEnum(enumManifestName, sceneLoading, "Scene Loading");
            EnumManifest.AddEnum(enumManifestName, vamUserSettings, "VAM User Settings");           
            EnumManifest.AddEnum(enumManifestName, vrSettings, "VR Settings");
            EnumManifest.AddEnum(enumManifestName, misc, "Miscellaneous");
            
        }

        public static string GetLabel(int buttonCategory)
        {
            return EnumManifest.GetDisplayVal(enumManifestName, buttonCategory);
        }

        public static int GetEnumVal(string buttonCatLabel)
        {
            return EnumManifest.GetEnumValFromDisplayVal(enumManifestName, buttonCatLabel);
        }

        public static List<string> displayValList
        {
            get
            {
                return (EnumManifest.GetDisplayValList(enumManifestName));
            }
        }
    }

    public static class BulkGridSetupButtonTypes
    {
        public static string enumManifestName
        {
            get { return "BulkGridSetupButtonTypes"; }
        }
        static BulkGridSetupButtonTypes()
        {
            EnumManifest.AddEnum(enumManifestName, UIAButtonOpType.loadAppPreset, "Load Appearance Preset");
            EnumManifest.AddEnum(enumManifestName, UIAButtonOpType.loadClothPreset, "Load Clothing Preset");
            EnumManifest.AddEnum(enumManifestName, UIAButtonOpType.loadPosePreset, "Load Pose Preset");
            EnumManifest.AddEnum(enumManifestName, UIAButtonOpType.loadHairPreset, "Load Hair Preset");
        }
    }
    public static class BulkGridSetupTargetTypes
    {
        public static string enumManifestName
        {
            get { return "BulkGridSetupTargetTypes"; }
        }
        static BulkGridSetupTargetTypes()
        {
            EnumManifest.AddEnum(enumManifestName, LastViewedTargetType.lastViewedFemale, "Last viewed Female");
            EnumManifest.AddEnum(enumManifestName, LastViewedTargetType.lastViewedMale, "Last viewed Male");
            EnumManifest.AddEnum(enumManifestName, LastViewedTargetType.lastViewedPerson, "Last viewed Person");
            EnumManifest.AddEnum(enumManifestName, UserChosenTargetType.femaleAtoms, "User chosen Female");
            EnumManifest.AddEnum(enumManifestName, UserChosenTargetType.maleAtoms, "User chosen Male");
            EnumManifest.AddEnum(enumManifestName, UserChosenTargetType.personAtoms, "User chosen Person");
        }
    }


    public static class UIAButtonOpType
    {
        public const int spawnAtom = 1;
        public const int toggleAtomOn = 2;
        public const int hideAtom = 3;
        public const int deleteAtom = 4;
        public const int selectAtom = 5;
        public const int moveAtom = 6;
        public const int teleportAtom = 7;
        public const int toggleAtomCollision = 8;       
        public const int togglePresetLocks = 10;
        public const int changeAtomName = 80;

        public const int detachAtomRoot = 9;
        public const int nodePositionState = 82;
        public const int nodeRotationState = 83;
        public const int toggleNodePositionState = 84;
        public const int toggleNodeRotationState = 85;
        public const int nodePhysics = 86;
        public const int toggleNodePhysics = 87;

        public const int resetScale = 11;
        public const int resetAppearance = 12;
        public const int decalMakerToggle = 13;
        public const int setHairColor = 14;
        public const int suppressScaleLoad = 15;
        public const int suppressClothingLoad = 79;

        public const int removeAllClothing = 16;
        public const int undressAllClothing = 17;
        public const int resetSimAllClothing = 18;
        public const int removeClothingPreset = 19;
        public const int mergeClothingPreset = 20;
        public const int undressClothingPreset = 21;
        public const int resetSimClothingPreset = 22;
        public const int activeClothingEditor = 23;

        public const int loadAppPreset = 24;
        public const int loadClothPreset = 25;
        public const int loadPosePreset = 26;
        public const int loadHairPreset = 27;
        public const int loadPluginsPreset = 28;
        public const int loadMorphPreset = 29;
        public const int loadSkinPreset = 30;
        public const int loadGlutePreset = 31;
        public const int loadBreastPreset = 32;
        public const int loadGeneralPreset = 33;

        public const int loadLegacyPreset = 34;
        public const int loadLegacyLook = 35;
        public const int loadLegacyPose = 36;
        public const int saveLegacyPreset = 37;
        public const int saveLegacyLook = 38;
        public const int saveLegacyPose = 39;

        public const int pluginAction = 40;
        public const int pluginActionToggle = 41;
        public const int pluginBoolToggle = 42;
        public const int pluginBoolTrue = 43;
        public const int pluginBoolFalse = 44;
        public const int pluginMultiSettings = 45;
        public const int pluginMultiSettingsToggle = 46;

        public const int loadPlugins = 47;
        public const int unLoadPlugins = 48;
        public const int pluginAssistLoad = 49;
        public const int pluginAssistFind = 50;

        public const int loadUIAProfile = 51;
        public const int gazeAssistedSelect = 52;
        public const int hideInactiveTargets = 53;
        public const int raiseByHAHeight = 54;
        public const int suppressPresetLocks = 55;
        public const int switchUIAGrid = 56;
        public const int hideUGameControlUI = 88;

        public const int startPlayback = 57;
        public const int stopPlayback = 58;
        public const int resetAnimation = 59;
        public const int toggleLoopPlayback = 60;
        public const int beginRecordMode = 61;
        public const int stopRecording = 62;
        public const int loadMotionCapture = 63;
        public const int loadELMotionCapture = 81;

        public const int freezeMotionSound = 64;
        public const int togglePlayEdit = 65;
        public const int setWorldScale = 66;
        public const int offsetWorldScale = 67;
        public const int teleportPlayer = 68;
        public const int blank = 69;

        public const int loadScene = 70;
        public const int mergeLoadScene = 71;
        public const int loadSubScene = 72;

        public const int softBodyPhysics = 73;
        public const int desktopVSync = 74;
        public const int perfMon = 75;
        public const int showHiddenAtoms = 76;
        public const int showVRHands = 77;
        public const int vrHeadCollider = 78;

        // Suppress Clothing Load is using enum val 79
        // Change Atom Name is using enum val 80
        // Load EverLaster MoCp is enum val 81
        // Node contol is enum val 82 to 87
        // GameControlUI Hide is 88

        public static string enumManifestName
        {
            get { return "UIAButtonType"; }
        }

        private static Dictionary<int, int> buttonTypeToCategory;

        private static Dictionary<int, List<int>> buttonTypesToFileReferenceTypes;

        private static Dictionary<int, List<int>> buttonTypesToComponentTypes;

        private static Dictionary<int, int> buttonTypesToMaxStates;

        private static Dictionary<int, string> buttonInactiveLabels;
        private static Dictionary<int, string> buttonActiveLabels;

        public static int GetMaxStates (int buttonType)
        {
            return buttonTypesToMaxStates[buttonType];
        }

        static UIAButtonOpType()
        {
            buttonTypeToCategory = new Dictionary<int, int>();
            buttonTypesToFileReferenceTypes = new Dictionary<int, List<int>>();
            buttonTypesToComponentTypes = new Dictionary<int, List<int>>();
            buttonTypesToMaxStates = new Dictionary<int, int>();
            buttonInactiveLabels = new Dictionary<int, string>();
            buttonActiveLabels = new Dictionary<int, string>();

            AddEnumeration(spawnAtom, UIAButtonCategory.atomControl, "Spawn Atom");
            AddLabel(spawnAtom, "Spawn <ATOMTYPE> at <RELATIVETARGET>");
            AddFileReferenceType(spawnAtom, FileReferenceTypes.appearancePreset);
            AddFileReferenceType(spawnAtom, FileReferenceTypes.posePreset);
            AddFileReferenceType(spawnAtom, FileReferenceTypes.pluginsPreset);
            AddFileReferenceType(spawnAtom, FileReferenceTypes.generalPreset);
            AddFileReferenceType(spawnAtom, FileReferenceTypes.subScene);
            AddButtonComponentTypes(spawnAtom, ButtonComponentTypes.spawnAtomComponent,ButtonComponentTypes.relativePositionComponent,ButtonComponentTypes.pluginsLoadComponent,ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(toggleAtomOn, UIAButtonCategory.atomControl, "Toggle Atom On/Off",null,2);
            AddLabel(toggleAtomOn, "Toggle Atom On", "Toggle Atom Off");
            AddButtonComponentTypes(toggleAtomOn, ButtonComponentTypes.targetComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(hideAtom, UIAButtonCategory.atomControl, "Hide Atom", null, 2);
            AddLabel(hideAtom, "Hide <TARGET>", "Show <TARGET>");
            AddButtonComponentTypes(hideAtom, ButtonComponentTypes.targetComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(deleteAtom, UIAButtonCategory.atomControl, "Delete Atom");
            AddLabel(deleteAtom, "Delete <TARGET>");
            AddButtonComponentTypes(deleteAtom, ButtonComponentTypes.targetComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(selectAtom, UIAButtonCategory.atomControl, "Select Atom");
            AddLabel(selectAtom, "Select <TARGET>");
            AddButtonComponentTypes(selectAtom, ButtonComponentTypes.targetComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(moveAtom, UIAButtonCategory.atomControl, "Move Atom (VR Only)", null, 2);
            AddLabel(moveAtom, "Move <TARGET>", "Stop Move");
            AddButtonComponentTypes(moveAtom, ButtonComponentTypes.targetComponent, ButtonComponentTypes.moveAtomComponent,ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(teleportAtom, UIAButtonCategory.atomControl, "Teleport Atom");
            AddLabel(teleportAtom, "Teleport <TARGET> to <RELATIVETARGET>");
            AddButtonComponentTypes(teleportAtom, ButtonComponentTypes.targetComponent, ButtonComponentTypes.relativePositionComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(toggleAtomCollision, UIAButtonCategory.atomControl, "Toggle Atom Collision", null, 2);
            AddLabel(toggleAtomCollision, "Enable Collision for <TARGET>", "Disable Collision for <TARGET>");
            AddButtonComponentTypes(toggleAtomCollision, ButtonComponentTypes.targetComponent, ButtonComponentTypes.buttonSkinComponent);
            
            AddEnumeration(togglePresetLocks, UIAButtonCategory.atomControl, "Toggle Preset Locks", null, 2);
            AddLabel(togglePresetLocks, "Activate Preset Locks for <TARGET>", "Deactivate Preset Locks for <TARGET>");
            AddButtonComponentTypes(togglePresetLocks, ButtonComponentTypes.targetComponent,ButtonComponentTypes.presetLockComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(changeAtomName, UIAButtonCategory.atomControl, "Change Atom Name", null, 1);
            AddLabel(changeAtomName, "Change Atom Name for <TARGET>");
            AddButtonComponentTypes(changeAtomName, ButtonComponentTypes.targetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(detachAtomRoot, UIAButtonCategory.nodeControl, "Detach Root Control", null, 2);
            AddLabel(detachAtomRoot, "Detach Root Control from <TARGET>", "Reattach Root Control to <TARGET>");
            AddButtonComponentTypes(detachAtomRoot, ButtonComponentTypes.targetComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(nodePositionState, UIAButtonCategory.nodeControl, "Set Position Control States", null, 1);
            AddLabel(nodePositionState, "Set Position Control States for <TARGET>");
            AddButtonComponentTypes(nodePositionState, ButtonComponentTypes.targetComponent, ButtonComponentTypes.nodeControlOnComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(toggleNodePositionState, UIAButtonCategory.nodeControl, "Toggle Position Control States", null, 2);
            AddLabel(toggleNodePositionState, "Set Position Control States for <TARGET>", "Unset Position Control States for <TARGET>");
            AddButtonComponentTypes(toggleNodePositionState, ButtonComponentTypes.targetComponent, ButtonComponentTypes.nodeControlOnComponent,
                ButtonComponentTypes.nodeControlOffComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(nodeRotationState, UIAButtonCategory.nodeControl, "Set Rotation Control States", null, 1);
            AddLabel(nodeRotationState, "Set Rotation Control States for <TARGET>");
            AddButtonComponentTypes(nodeRotationState, ButtonComponentTypes.targetComponent, ButtonComponentTypes.nodeControlOnComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(toggleNodeRotationState, UIAButtonCategory.nodeControl, "Toggle Rotation Control States", null, 2);
            AddLabel(toggleNodeRotationState, "Set Rotation Control States for <TARGET>", "Unset Rotation Control States for <TARGET>");
            AddButtonComponentTypes(toggleNodeRotationState, ButtonComponentTypes.targetComponent, ButtonComponentTypes.nodeControlOnComponent, ButtonComponentTypes.nodeControlOffComponent, ButtonComponentTypes.buttonSkinComponent);


            AddEnumeration(nodePhysics, UIAButtonCategory.nodeControl, "Set Node Physics", null, 1);
            AddLabel(nodePhysics, "Set Node Physics for <TARGET>");
            AddButtonComponentTypes(nodePhysics, ButtonComponentTypes.targetComponent, ButtonComponentTypes.nodePhysicsOnComponent, ButtonComponentTypes.nodeSelectionComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(toggleNodePhysics, UIAButtonCategory.nodeControl, "Toggle Node Physics", null, 2);
            AddLabel(toggleNodePhysics, "Set Node Physics for <TARGET>", "Unset Node Physics for <TARGET>");
            AddButtonComponentTypes(toggleNodePhysics, ButtonComponentTypes.targetComponent, ButtonComponentTypes.nodePhysicsOnComponent, 
                ButtonComponentTypes.nodePhysicsOffComponent, ButtonComponentTypes.nodeSelectionComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(resetScale, UIAButtonCategory.appearance, "Reset Scale");
            AddLabel(resetScale, "Reset Scale for <TARGET>");
            AddButtonComponentTypes(resetScale, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(resetAppearance, UIAButtonCategory.appearance, "Reset Appearance");
            AddLabel(resetAppearance, "Reset Appearance for <TARGET>");
            AddButtonComponentTypes(resetAppearance, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(decalMakerToggle, UIAButtonCategory.appearance, "Decal Maker Toggle", null, 2);
            AddLabel(decalMakerToggle, "Toggle Decals On for <TARGET>", "Toggle Decals Off for <TARGET>");
            AddButtonComponentTypes(decalMakerToggle, ButtonComponentTypes.targetComponent, ButtonComponentTypes.decalMakerComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(setHairColor, UIAButtonCategory.appearance, "Set Hair Color");
            AddLabel(setHairColor, "Set Hair color for <TARGET>");
            AddButtonComponentTypes(setHairColor, ButtonComponentTypes.targetComponent, ButtonComponentTypes.hairColorComponent, ButtonComponentTypes.buttonSkinComponent);
            if (PatreonFeatures.patreonContentEnabled)
            {
                AddEnumeration(suppressScaleLoad, UIAButtonCategory.appearance, "Suppress Scale Load", null, 2);
                AddLabel(suppressScaleLoad, "Enable Suppress Scale Load", "Disable Suppress Scale Load");
                AddButtonComponentTypes(suppressScaleLoad, ButtonComponentTypes.buttonSkinComponent);
            }
            AddEnumeration(suppressClothingLoad, UIAButtonCategory.appearance, "Suppress Clothing Load", null, 2);
            AddLabel(suppressClothingLoad, "Enable Suppress Clothing Load", "Disable Suppress Clothing Load");
            AddButtonComponentTypes(suppressClothingLoad, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(removeAllClothing, UIAButtonCategory.clothing, "Remove All Clothing");
            AddLabel(removeAllClothing, "Remove all Clothing for <TARGET>");
            AddButtonComponentTypes(removeAllClothing, ButtonComponentTypes.targetComponent,                ButtonComponentTypes.clothingComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(undressAllClothing, UIAButtonCategory.clothing, "Undress All Clothing", null, 2);
            AddLabel(undressAllClothing, "Undress all Clothing for <TARGET>", "Dress All Clothing for <TARGET>");
            AddButtonComponentTypes(undressAllClothing, ButtonComponentTypes.targetComponent, ButtonComponentTypes.clothingComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(resetSimAllClothing, UIAButtonCategory.clothing, "Reset Sim All Clothing");
            AddLabel(resetSimAllClothing, "Reset Sim on all Clothing for <TARGET>");
            AddButtonComponentTypes(resetSimAllClothing, ButtonComponentTypes.targetComponent, ButtonComponentTypes.clothingComponent, ButtonComponentTypes.buttonSkinComponent);
            if (PatreonFeatures.patreonContentEnabled)
            {
                AddEnumeration(removeClothingPreset, UIAButtonCategory.clothing, "Remove Clothing Preset");
                AddLabel(removeClothingPreset, "Remove <FILEREF1> clothing for <TARGET>");
                AddFileReferenceType(removeClothingPreset, FileReferenceTypes.clothingPreset);
                AddButtonComponentTypes(removeClothingPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.clothingComponent, ButtonComponentTypes.buttonSkinComponent);

                AddEnumeration(mergeClothingPreset, UIAButtonCategory.clothing, "Merge Clothing Preset", null, 2);
                AddLabel(mergeClothingPreset, "Merge <FILEREF1> clothing for <TARGET>", "Unmerge <FILEREF1> clothing for <TARGET>");
                AddFileReferenceType(mergeClothingPreset, FileReferenceTypes.clothingPreset);
                AddButtonComponentTypes(mergeClothingPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.clothingComponent, ButtonComponentTypes.buttonSkinComponent);

                AddEnumeration(undressClothingPreset, UIAButtonCategory.clothing, "Undress Cloth Preset", "Undress Clothing Preset",2);
                AddLabel(undressClothingPreset, "Undress <FILEREF1> clothing for <TARGET>", "Dress <FILEREF1> clothing for <TARGET>");
                AddFileReferenceType(undressClothingPreset, FileReferenceTypes.clothingPreset);
                AddButtonComponentTypes(undressClothingPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.clothingComponent, ButtonComponentTypes.buttonSkinComponent);

                AddEnumeration(resetSimClothingPreset, UIAButtonCategory.clothing, "Reset Sim Cloth Preset", "Reset Sim Clothing Preset");
                AddLabel(resetSimClothingPreset, "Sim Reset <FILEREF1> clothing for <TARGET>");
                AddFileReferenceType(resetSimClothingPreset, FileReferenceTypes.clothingPreset);
                AddButtonComponentTypes(resetSimClothingPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.clothingComponent, ButtonComponentTypes.buttonSkinComponent);

                AddEnumeration(activeClothingEditor, UIAButtonCategory.clothing, "Active Clothing Editor", null, 2);
                AddLabel(activeClothingEditor, "Open Active Clothing", "Close Active Clothing");
                AddButtonComponentTypes(activeClothingEditor, ButtonComponentTypes.targetComponent, ButtonComponentTypes.buttonSkinComponent);
            }

            AddEnumeration(loadAppPreset, UIAButtonCategory.presets, "Load App Preset", "Load Appearance Preset");
            AddLabel(loadAppPreset, "Load Appearance <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadAppPreset, FileReferenceTypes.appearancePreset);
            AddButtonComponentTypes(loadAppPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadClothPreset, UIAButtonCategory.presets, "Load Clothing Preset");
            AddLabel(loadClothPreset, "Load Clothing <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadClothPreset, FileReferenceTypes.clothingPreset);
            AddButtonComponentTypes(loadClothPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadPosePreset, UIAButtonCategory.presets, "Load Pose Preset");
            AddLabel(loadPosePreset, "Load Pose <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadPosePreset, FileReferenceTypes.posePreset);
            AddButtonComponentTypes(loadPosePreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadHairPreset, UIAButtonCategory.presets, "Load Hair Preset");
            AddLabel(loadHairPreset, "Load Hair <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadHairPreset, FileReferenceTypes.hairPreset);
            AddButtonComponentTypes(loadHairPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadPluginsPreset, UIAButtonCategory.presets, "Load Plugins Preset");
            AddLabel(loadPluginsPreset, "Load Plugins <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadPluginsPreset, FileReferenceTypes.pluginsPreset);
            AddButtonComponentTypes(loadPluginsPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadMorphPreset, UIAButtonCategory.presets, "Load Morph Preset");
            AddLabel(loadMorphPreset, "Load Morph <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadMorphPreset, FileReferenceTypes.morphPreset);
            AddButtonComponentTypes(loadMorphPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadSkinPreset, UIAButtonCategory.presets, "Load Skin Preset");
            AddLabel(loadSkinPreset, "Load Skin <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadSkinPreset, FileReferenceTypes.skinPreset);
            AddButtonComponentTypes(loadSkinPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.skinPresetDecalComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadGlutePreset, UIAButtonCategory.presets, "Load Glute Preset");
            AddLabel(loadGlutePreset, "Load Glute Physics <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadGlutePreset, FileReferenceTypes.glutePreset);
            AddButtonComponentTypes(loadGlutePreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadBreastPreset, UIAButtonCategory.presets, "Load Breast Preset");
            AddLabel(loadBreastPreset, "Load Breast Physics <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadBreastPreset, FileReferenceTypes.breastPreset);
            AddButtonComponentTypes(loadBreastPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadGeneralPreset, UIAButtonCategory.presets, "Load General Preset");
            AddLabel(loadGeneralPreset, "Load General Preset <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadGeneralPreset, FileReferenceTypes.generalPreset);
            AddButtonComponentTypes(loadGeneralPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadLegacyPreset, UIAButtonCategory.legacyPresets, "Load Legacy Preset");
            AddLabel(loadLegacyPreset, "Load Legacy Preset <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadLegacyPreset, FileReferenceTypes.legacyPreset);
            AddButtonComponentTypes(loadLegacyPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadLegacyLook, UIAButtonCategory.legacyPresets, "Load Legacy Appearance", "Load Legacy Look");
            AddLabel(loadLegacyLook, "Load Legacy Look <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadLegacyLook, FileReferenceTypes.legacyLookPreset);
            AddButtonComponentTypes(loadLegacyLook, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadLegacyPose, UIAButtonCategory.legacyPresets, "Load Legacy Pose");
            AddLabel(loadLegacyPose, "Load Legacy Pose <FILEREF1> to <TARGET>");
            AddFileReferenceType(loadLegacyPose, FileReferenceTypes.legacyPosePreset);
            AddButtonComponentTypes(loadLegacyPose, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(saveLegacyPreset, UIAButtonCategory.legacyPresets, "Save Legacy Preset");
            AddLabel(saveLegacyPreset, "Save Legacy Preset <FILEREF1> to <TARGET>");
            AddFileReferenceType(saveLegacyPreset, FileReferenceTypes.legacyPreset);
            AddButtonComponentTypes(saveLegacyPreset, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(saveLegacyLook, UIAButtonCategory.legacyPresets, "Save Legacy Appearance", "Save Legacy Look");
            AddLabel(saveLegacyLook, "Save Legacy Look <FILEREF1> to <TARGET>");
            AddFileReferenceType(saveLegacyLook, FileReferenceTypes.legacyLookPreset);
            AddButtonComponentTypes(saveLegacyLook, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(saveLegacyPose, UIAButtonCategory.legacyPresets, "Save Legacy Pose");
            AddLabel(saveLegacyPose, "Save Legacy Pose <FILEREF1> to <TARGET>");
            AddFileReferenceType(saveLegacyPose, FileReferenceTypes.legacyPosePreset);
            AddButtonComponentTypes(saveLegacyPose, ButtonComponentTypes.targetComponent, ButtonComponentTypes.appearancePresetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(pluginAction, UIAButtonCategory.pluginSettings, "Plugin Action");
            AddLabel(pluginAction, "<PLUGINTYPE>: <PARAMON> for <TARGET>");
            AddButtonComponentTypes(pluginAction, ButtonComponentTypes.targetComponent, ButtonComponentTypes.pluginSettingComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(pluginActionToggle, UIAButtonCategory.pluginSettings, "Plugin Action Toggle",null,2);
            AddLabel(pluginActionToggle, "<PLUGINTYPE>: <PARAMON> for <TARGET>", "<PLUGINTYPE>: <PARAMOFF> for <TARGET>");
            AddButtonComponentTypes(pluginActionToggle, ButtonComponentTypes.targetComponent, ButtonComponentTypes.pluginSettingComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(pluginBoolToggle, UIAButtonCategory.pluginSettings, "Plugin Bool Toggle", null, 2);
            AddLabel(pluginBoolToggle, "Set <PLUGINTYPE>: <PARAMON> to TRUE for <TARGET>", "Set <PLUGINTYPE>: <PARAMON> to FALSE for <TARGET>");
            AddButtonComponentTypes(pluginBoolToggle, ButtonComponentTypes.targetComponent, ButtonComponentTypes.pluginSettingComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(pluginBoolTrue, UIAButtonCategory.pluginSettings, "Set Plugin Bool True");
            AddLabel(pluginBoolTrue, "Set <PLUGINTYPE>: <PARAMON> to TRUE for <TARGET>");
            AddButtonComponentTypes(pluginBoolTrue, ButtonComponentTypes.targetComponent, ButtonComponentTypes.pluginSettingComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(pluginBoolFalse, UIAButtonCategory.pluginSettings, "Set Plugin Bool False");
            AddLabel(pluginBoolFalse, "Set <PLUGINTYPE>: <PARAMON> to FALSE for <TARGET>");
            AddButtonComponentTypes(pluginBoolFalse, ButtonComponentTypes.targetComponent, ButtonComponentTypes.pluginSettingComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(pluginMultiSettings, UIAButtonCategory.pluginSettings, "Set Plugin Settings");
            AddLabel(pluginMultiSettings, "<PLUGINTYPE>: Settings for <TARGET>");
            AddButtonComponentTypes(pluginMultiSettings, ButtonComponentTypes.targetComponent, ButtonComponentTypes.pluginSettingComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(pluginMultiSettingsToggle, UIAButtonCategory.pluginSettings, "Toggle Plugin Settings", null, 2);
            AddLabel(pluginMultiSettingsToggle, "<PLUGINTYPE>: Settings ON for <TARGET>", "<PLUGINTYPE>: Settings OFF for <TARGET>");
            AddButtonComponentTypes(pluginMultiSettingsToggle, ButtonComponentTypes.targetComponent, ButtonComponentTypes.pluginSettingComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadPlugins, UIAButtonCategory.pluginsLoading, "Load Plugin(s)");
            AddLabel(loadPlugins, "Load Plugins to <TARGET>");
            AddButtonComponentTypes(loadPlugins, ButtonComponentTypes.targetComponent, ButtonComponentTypes.pluginsLoadComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(unLoadPlugins, UIAButtonCategory.pluginsLoading, "Unload All Plugins");
            AddLabel(unLoadPlugins, "Unload all Plugins from <TARGET>");
            AddButtonComponentTypes(unLoadPlugins, ButtonComponentTypes.targetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(pluginAssistLoad, UIAButtonCategory.pluginsLoading, "Open PluginAssist Load");
            AddLabel(pluginAssistLoad, "Open PluginAssist Load");
            AddButtonComponentTypes(pluginAssistLoad, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(pluginAssistFind, UIAButtonCategory.pluginsLoading, "Open PluginAssist Find");
            AddLabel(pluginAssistFind, "Open PluginAssist Find");
            AddButtonComponentTypes(pluginAssistFind, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadUIAProfile, UIAButtonCategory.uiAssist, "Load UI Assist Profile");
            AddLabel(loadUIAProfile, "Load UIA Profile <FILEREF1>");
            AddFileReferenceType(loadUIAProfile, FileReferenceTypes.uiap);
            AddButtonComponentTypes(loadUIAProfile, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(gazeAssistedSelect, UIAButtonCategory.uiAssist, "Gaze Assisted Select",null,2);
            AddLabel(gazeAssistedSelect, "Gaze Assisted Select ON", "Gaze Assisted Select OFF");
            AddButtonComponentTypes(gazeAssistedSelect, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(hideInactiveTargets, UIAButtonCategory.vamUserSettings, "Hide Inactive Targets", null, 2);
            AddLabel(hideInactiveTargets, "Hide Inactive Targets", "Show Inactive Targets");
            AddButtonComponentTypes(hideInactiveTargets, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(raiseByHAHeight, UIAButtonCategory.uiAssist, "Raise People by Heel Adjust Height", null, 2);
            AddLabel(raiseByHAHeight, "Enable Heel Adjust Height raise", "Disable Heel Adjust Height raise");
            AddButtonComponentTypes(raiseByHAHeight, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(suppressPresetLocks, UIAButtonCategory.uiAssist, "Suppress Preset Locks on UIA Preset Loads", null, 2);
            AddLabel(suppressPresetLocks, "Suppress Preset Locks", "Unsuppress Preset Locks");
            AddButtonComponentTypes(suppressPresetLocks, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(switchUIAGrid, UIAButtonCategory.uiAssist, "Switch UIA Screen","Switch UIA Grid");
            AddLabel(switchUIAGrid, "Switch to Grid <UIASCREEN>");
            AddButtonComponentTypes(switchUIAGrid,ButtonComponentTypes.switchUIAGridComponent , ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(hideUGameControlUI, UIAButtonCategory.uiAssist, "Hide Game Control UI", "Hide Game Control UI");
            AddLabel(hideUGameControlUI, "Hide Game Control UI");
            AddButtonComponentTypes(hideUGameControlUI, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(startPlayback, UIAButtonCategory.sceneAnim, "Start Playback");
            AddLabel(startPlayback, "Play Scene Mocap Animation");
            AddButtonComponentTypes(startPlayback, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(stopPlayback, UIAButtonCategory.sceneAnim, "Stop Playback");
            AddLabel(stopPlayback, "Stop Scene Mocap Animation");
            AddButtonComponentTypes(stopPlayback, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(resetAnimation, UIAButtonCategory.sceneAnim, "Reset Animation");
            AddLabel(resetAnimation, "Reset Scene Mocap Animation");
            AddButtonComponentTypes(resetAnimation, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(toggleLoopPlayback, UIAButtonCategory.sceneAnim, "Toggle Loop Playback",null,2);
            AddLabel(toggleLoopPlayback, "Enable Scene Mocap Loop", "Disable Scene Mocap Loop");
            AddButtonComponentTypes(toggleLoopPlayback, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(beginRecordMode, UIAButtonCategory.sceneAnim, "Begin Record Mode");
            AddLabel(beginRecordMode, "Begin Scene Mocap Record");
            AddButtonComponentTypes(beginRecordMode, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(stopRecording, UIAButtonCategory.sceneAnim, "Stop Recording");
            AddLabel(stopRecording, "Stop Scene Mocap Record");
            AddButtonComponentTypes(stopRecording, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(loadMotionCapture, UIAButtonCategory.sceneAnim, "Load Motion Capture from Scene");
            AddLabel(loadMotionCapture, "Load Mocap from <FILEREF1>");
            AddFileReferenceType(loadMotionCapture, FileReferenceTypes.scene);
            AddButtonComponentTypes(loadMotionCapture, ButtonComponentTypes.targetComponent, ButtonComponentTypes.motionCaptureComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(loadELMotionCapture, UIAButtonCategory.sceneAnim, "Load EverLaster Mocap");
            AddLabel(loadELMotionCapture, "Load Mocap from <FILEREF1>");
            AddFileReferenceType(loadELMotionCapture, FileReferenceTypes.mocap);
            AddButtonComponentTypes(loadELMotionCapture, ButtonComponentTypes.targetComponent, ButtonComponentTypes.motionCaptureComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(freezeMotionSound, UIAButtonCategory.misc, "Freeze Motion/Sound",null,2);
            AddLabel(freezeMotionSound, "Freeze Motion / Sound", "Unfreeze Motion / Sound");
            AddButtonComponentTypes(freezeMotionSound, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(togglePlayEdit, UIAButtonCategory.misc, "Toggle Play/Edit Mode",null,2);
            AddLabel(togglePlayEdit, "Edit Mode", "Play Mode");
            AddButtonComponentTypes(togglePlayEdit, ButtonComponentTypes.vamPlayEditModeComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(setWorldScale, UIAButtonCategory.misc, "Set World Scale");
            AddLabel(setWorldScale, "Set World Scale to <WORLDSCALE>");
            AddButtonComponentTypes(setWorldScale, ButtonComponentTypes.worldScaleComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(offsetWorldScale, UIAButtonCategory.misc, "Offset World Scale");
            AddButtonComponentTypes(offsetWorldScale, ButtonComponentTypes.worldScaleComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(teleportPlayer, UIAButtonCategory.misc, "Teleport Player");
            AddLabel(teleportPlayer, "Teleport Player to <RELATIVETARGET>");
            AddButtonComponentTypes(teleportPlayer, ButtonComponentTypes.relativePositionComponent, ButtonComponentTypes.buttonSkinComponent);
            AddEnumeration(blank, UIAButtonCategory.misc, "Blank");
            AddLabel(blank, "");

            AddEnumeration(loadScene, UIAButtonCategory.sceneLoading, "Load Scene");
            AddLabel(loadScene, "Load Scene <FILEREF1>");
            AddFileReferenceType(loadScene, FileReferenceTypes.scene);
            AddFileReferenceType(loadScene, FileReferenceTypes.addonPackagesFolder);
            AddButtonComponentTypes(loadScene, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(mergeLoadScene, UIAButtonCategory.sceneLoading, "Merge Load Scene");
            AddLabel(mergeLoadScene, "Merge Load Scene <FILEREF1>");
            AddFileReferenceType(mergeLoadScene, FileReferenceTypes.scene);
            AddFileReferenceType(mergeLoadScene, FileReferenceTypes.addonPackagesFolder);
            AddButtonComponentTypes(mergeLoadScene, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(loadSubScene, UIAButtonCategory.sceneLoading, "Load SubScene");
            AddLabel(loadSubScene, "Load SubScene <FILEREF1>");
            AddFileReferenceType(loadSubScene, FileReferenceTypes.subScene);
            AddButtonComponentTypes(loadSubScene, ButtonComponentTypes.targetComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(softBodyPhysics, UIAButtonCategory.vamUserSettings, "Toggle Soft Body Physics",null,2);
            AddLabel(softBodyPhysics, "Enable Soft Body Physics", "Disable Soft Body Physics");
            AddButtonComponentTypes(softBodyPhysics, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(desktopVSync, UIAButtonCategory.vamUserSettings, "Toggle Desktop VSync", null, 2);
            AddLabel(desktopVSync, "Enable Desktop VSync", "Disable Desktop VSync");
            AddButtonComponentTypes(desktopVSync, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(perfMon, UIAButtonCategory.vamUserSettings, "Toggle Performance Monitor", null, 2);
            AddLabel(perfMon, "Enable Perf Monitor", "Disable Perf Monitor");
            AddButtonComponentTypes(perfMon, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(showHiddenAtoms, UIAButtonCategory.vamUserSettings, "Toggle Show Hidden Atoms", null, 2);
            AddLabel(showHiddenAtoms, "Show Hidden Atoms", "Disable Show Hidden Atoms");
            AddButtonComponentTypes(showHiddenAtoms, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(showVRHands, UIAButtonCategory.vrSettings, "Toggle Show VR Hands", null, 2);
            AddLabel(showVRHands, "Show VR Hands", "Hide VR Hands");
            AddButtonComponentTypes(showVRHands, ButtonComponentTypes.showVRHandsComponent, ButtonComponentTypes.buttonSkinComponent);

            AddEnumeration(vrHeadCollider, UIAButtonCategory.vrSettings, "Toggle VR Head Collider", null, 2);
            AddLabel(vrHeadCollider, "Enable VR Head Collider", "Disable VR Head Collider");
            AddButtonComponentTypes(vrHeadCollider, ButtonComponentTypes.buttonSkinComponent);
        }

        private static void AddEnumeration(int enumeration, int buttonCat, string storeValLabel, string altLabel = null, int maxStates=1)
        {
            EnumManifest.AddEnum(enumManifestName, enumeration, storeValLabel, altLabel);
            buttonTypeToCategory.Add(enumeration, buttonCat);
            buttonTypesToMaxStates.Add(enumeration, maxStates);
        }
        private static void AddFileReferenceType(int enumeration, int fileRefType)
        {
            if (!buttonTypesToFileReferenceTypes.ContainsKey(enumeration)) buttonTypesToFileReferenceTypes.Add(enumeration, new List<int>());
            if (!buttonTypesToFileReferenceTypes[enumeration].Contains(fileRefType)) buttonTypesToFileReferenceTypes[enumeration].Add(fileRefType);
        }

        private static void AddButtonComponentTypes(int enumeration, params int[] componentTypes)
        {
            if (!buttonTypesToComponentTypes.ContainsKey(enumeration)) buttonTypesToComponentTypes.Add(enumeration, new List<int>());
            foreach (int componentType in componentTypes)
            {
                buttonTypesToComponentTypes[enumeration].Add(componentType);
            }
        }

        private static void AddLabel(int enumeration, string inactiveLabel, string activeLabel = "")
        {
            if (buttonInactiveLabels.ContainsKey(enumeration)) SuperController.LogError("UIAssist: duplicate button labels registered for button type " + enumeration.ToString()) ;
            buttonInactiveLabels.Add(enumeration, inactiveLabel);
            if (buttonTypesToMaxStates[enumeration] > 1)
            {
                if (activeLabel != "") buttonActiveLabels.Add(enumeration, activeLabel);
                else SuperController.LogError("UIAssist: No Active label set for button type " + enumeration.ToString());
            }
            
        }

        public static string GetLabel(int buttonType)
        {
            return EnumManifest.GetDisplayVal(enumManifestName, buttonType);
        }
        public static int GetEnumVal(string buttonTypeLabel)
        {
            return EnumManifest.GetEnumValFromDisplayVal(enumManifestName, buttonTypeLabel);
        }
        public static List<string> displayValsList
        {
            get
            {
                return (EnumManifest.GetDisplayValList(enumManifestName));
            }
        }

        public static int GetButtonCategory(int buttonType)
        {
            if (!buttonTypeToCategory.ContainsKey(buttonType)) SuperController.LogError("UIAssist: Invalid enumeration in UIAButtonType.GetButtonCategory - " + buttonType.ToString());
            return buttonTypeToCategory[buttonType];
        }

        public static List<string> GetButtonTypeList(int buttonCat)
        {
            List<string> buttonTypes = new List<string>();
            foreach (KeyValuePair<int, int> kvp in buttonTypeToCategory)
            {
                if (kvp.Value == buttonCat) buttonTypes.Add(EnumManifest.GetDisplayVal(UIAButtonOpType.enumManifestName, kvp.Key));
            }

            return buttonTypes;
        }

        public static string GetInactiveLabel(int _buttonType)
        {
            if (!buttonInactiveLabels.ContainsKey(_buttonType)) SuperController.LogError("UIA.UIAButtonType.GetInactiveLabel: Missing inactive label for button type"+_buttonType.ToString());
            return buttonInactiveLabels[_buttonType];
        }
        public static string GetActiveLabel(int _buttonType)
        {
            if (!buttonActiveLabels.ContainsKey(_buttonType)) SuperController.LogError("UIA.UIAButtonType.GetActiveLabel: Missing active label for button type" + _buttonType.ToString());
            return buttonActiveLabels[_buttonType];
        }

        public static List<int> GetButtonTypeExclusions(int buttonCatInclusion)
        {
            List<int> exclusionList = new List<int>();
            foreach (KeyValuePair<int, int> kvp in buttonTypeToCategory)
            {
                if (kvp.Value != buttonCatInclusion) exclusionList.Add(kvp.Key);
            }

            return exclusionList;
        }

        public static List<int> GetFileReferenceTypes(int _buttonType)
        {
            List<int> fileRefTypes = new List<int>();
            if (buttonTypesToFileReferenceTypes.ContainsKey(_buttonType)) return buttonTypesToFileReferenceTypes[_buttonType];
            return fileRefTypes;
        }

        public static bool IsLegacyPresetLoadSaveType(int _buttonType)
        {
            return (GetButtonCategory(_buttonType) == UIAButtonCategory.legacyPresets);
        }
        public static bool IsLegacyPresetSaveType(int _buttonType)
        {
            return (_buttonType == saveLegacyPreset || _buttonType == saveLegacyLook || _buttonType == saveLegacyPose);
        }
        public static bool IsNonLegacyPresetLoadType(int _buttonType)
        {
            return GetButtonCategory(_buttonType) == UIAButtonCategory.presets;
        }

        public static bool IsClothingPresetType(int _buttonType)
        {
            return _buttonType == removeClothingPreset || _buttonType == mergeClothingPreset || _buttonType == undressClothingPreset || _buttonType == resetSimClothingPreset;
        }

        public static bool IsPresetType(int _buttonType)
        {
            return IsLegacyPresetLoadSaveType(_buttonType) || IsNonLegacyPresetLoadType(_buttonType) || IsClothingPresetType(_buttonType);
        }

        public static bool IsNonPersonAtomTargetableType(int _buttonType)
        {
            return _buttonType==toggleAtomOn || _buttonType == hideAtom || _buttonType == deleteAtom || _buttonType == selectAtom || _buttonType == loadSubScene || _buttonType == moveAtom || _buttonType == teleportAtom || _buttonType == toggleAtomCollision || _buttonType==loadGeneralPreset ||GetButtonCategory(_buttonType)== UIAButtonCategory.pluginSettings || GetButtonCategory(_buttonType) == UIAButtonCategory.pluginsLoading || _buttonType==changeAtomName;
        }

        public static bool IsClothingPresetLoadType(int _buttonType)
        {
            return (_buttonType == removeClothingPreset || _buttonType == mergeClothingPreset || _buttonType == undressClothingPreset || _buttonType == resetSimClothingPreset || _buttonType == loadClothPreset);
        }

        public static bool IsUIAClothingPresetLoadType(int _buttonType)
        {
            return (_buttonType == removeClothingPreset || _buttonType == mergeClothingPreset || _buttonType == undressClothingPreset || _buttonType == resetSimClothingPreset);
        }

        public static bool IsSceneLoadType(int _buttonType)
        {
            return _buttonType == loadScene || _buttonType == loadMotionCapture || _buttonType == mergeLoadScene;
        }
        public static bool IsUIAPLoadType(int _buttonType)
        {
            return _buttonType == loadUIAProfile;
        }

        //Button is of a toggle type (e.g. 2 states) and uses a preset file
        public static bool IsPresetToggleType(int _buttonType)
        {
            return _buttonType == undressClothingPreset || _buttonType == mergeClothingPreset || _buttonType == undressClothingPreset;

        }

        public static bool IsAtomTargetable(int _buttonType)
        {
            return ((_buttonType != suppressScaleLoad  &&
                GetButtonCategory(_buttonType) != UIAButtonCategory.misc &&
                GetButtonCategory(_buttonType) != UIAButtonCategory.uiAssist &&
                GetButtonCategory(_buttonType) != UIAButtonCategory.sceneAnim &&
                _buttonType != pluginAssistLoad &&
                _buttonType != pluginAssistFind) ||
                _buttonType == loadMotionCapture ||
                _buttonType == loadELMotionCapture);

        }

        public static string GetLegacyUIAPFileStoreNumber(int _buttonType, int fileRefType)
        {
            string fileStoreNumber = "1";
            if (fileRefType == FileReferenceTypes.addonPackagesFolder) fileStoreNumber = "2";
            if (_buttonType == spawnAtom && fileRefType == FileReferenceTypes.posePreset) fileStoreNumber = "3";
            if (_buttonType == spawnAtom && fileRefType == FileReferenceTypes.pluginsPreset) fileStoreNumber = "-1";
            return (fileStoreNumber);
        }
        public static int GetFileRefTypeFromLegacyUIAPFileStoreNumber(int _buttonType, int legacyFileRef,int spawnAtomType=-1)
        {
            if (!buttonTypesToFileReferenceTypes.ContainsKey(_buttonType)) return -1;

            if ((_buttonType == loadScene || _buttonType == mergeLoadScene) && legacyFileRef==2) return(FileReferenceTypes.addonPackagesFolder);

            if (_buttonType == spawnAtom)
            {
                if (legacyFileRef == 3) return FileReferenceTypes.posePreset;
                if (legacyFileRef == 1)
                {
                    if (spawnAtomType == AtomTypes.person) return FileReferenceTypes.appearancePreset;
                    else if (spawnAtomType == AtomTypes.subScene) return FileReferenceTypes.subScene;
                    else return FileReferenceTypes.generalPreset;
                }
            }

            return buttonTypesToFileReferenceTypes[_buttonType].First();
        }

        public static bool IsComponentInButtonType(int buttonType, int componentType)
        {
            if (!buttonTypesToComponentTypes.ContainsKey(buttonType)) return false;
            return (buttonTypesToComponentTypes[buttonType].Contains(componentType));
        }

        public static List<int> GetComponentTypes(int buttonType)
        {
            if (!buttonTypesToComponentTypes.ContainsKey(buttonType)) return new List<int>();
            return (buttonTypesToComponentTypes[buttonType]);
        }
    }



    public static class TargetCategory
    {
        public const int gazeSelectedAtom = 1;
        public const int vamSelectedAtom = 2;
        public const int atomGroup = 3;
        public const int specificAtom = 4;
        public const int userChosenAtom = 5;
        public const int scenePlugins = 6;
        public const int sessionPlugins = 7;

        public static string enumManifestName
        {
            get { return "TargetCategory"; }
        }

        static TargetCategory()
        {
            EnumManifest.AddEnum(enumManifestName, gazeSelectedAtom, "Gaze Selected Atom");
            EnumManifest.AddEnum(enumManifestName, vamSelectedAtom, "VAM Selected Atom");
            EnumManifest.AddEnum(enumManifestName, atomGroup, "Atom Group");
            EnumManifest.AddEnum(enumManifestName, specificAtom, "Specific Atom");
            EnumManifest.AddEnum(enumManifestName, userChosenAtom, "User Chosen Atom");
            EnumManifest.AddEnum(enumManifestName, scenePlugins, "Scene Plugins");
            EnumManifest.AddEnum(enumManifestName, sessionPlugins, "Session Plugins");
        }

        public static string GetLabel(int targetCategory)
        {
            return EnumManifest.GetDisplayVal(enumManifestName, targetCategory);
        }

        public static int GetEnumVal(string buttonCatLabel)
        {
            return EnumManifest.GetEnumValFromDisplayVal(enumManifestName, buttonCatLabel);
        }

        public static List<string> displayValList
        {
            get
            {
                return (EnumManifest.GetDisplayValList(enumManifestName));
            }
        }
    }

    public static class FileSelectionMode
    {
        public const int singleFile = 1;
        public const int randomFromFolder = 2;
        public const int alphabeticalFromFolder = 3;
        public const int chooseFromFolder = 4;
        public const int none = 5;

        public static string enumManifestName
        {
            get { return "FileSelectionMode"; }
        }

        static FileSelectionMode()
        {
            EnumManifest.AddEnum(enumManifestName, singleFile, "Single File");
            EnumManifest.AddEnum(enumManifestName, randomFromFolder, "Random from Folder");
            EnumManifest.AddEnum(enumManifestName, alphabeticalFromFolder, "Alphabetical filename sequence from Folder");
            EnumManifest.AddEnum(enumManifestName, chooseFromFolder, "Choose from Folder");
            EnumManifest.AddEnum(enumManifestName, none, "None");
        }

        public static string GetDisplayVal(int fileSelectMode)
        {
            return EnumManifest.GetDisplayVal(enumManifestName, fileSelectMode);
        }

        public static int GetEnumVal(string fileSelectModeLabel)
        {
            return EnumManifest.GetEnumValFromDisplayVal(enumManifestName, fileSelectModeLabel);
        }

        public static List<string> displayValList
        {
            get
            {
                return (EnumManifest.GetDisplayValList(enumManifestName));
            }
        }
    }
    public static class NodePhysicsModificationMode
    {
        public const int noChange = 1;
        public const int defaultValue = 2;
        public const int customValue = 3;
        public static string enumManifestName
        {
            get { return "NodePhysicsModificationMode"; }
        }

        static NodePhysicsModificationMode()
        {
            EnumManifest.AddEnum(enumManifestName, noChange, "Do not modify");
            EnumManifest.AddEnum(enumManifestName, defaultValue,"Default", "Set to Default");
            EnumManifest.AddEnum(enumManifestName, customValue, "Custom", "Set to custom value...");
        }
    }
    public static class NodeBodyRegion
    {
        public const int head = 1;
        public const int torso = 2;
        public const int gens = 3;
        public const int limbs = 4;
    }
    public static class NodePhysicsType
    {
        public const int hold = 1;
        public const int link = 2;
        public const int compliance = 3;
        public const int jointDrive = 4;
        public const int maxVelocity = 5;
    }

    public static class NodeControlState
    {
        public const int on = 1;
        public const int comply = 2;
        public const int off = 3;
        public const int parentLink = 4;
        public const int physicsLink = 5;
        public const int hold = 6;
        public const int lockNode = 7;

        public const int noChange = 10;

        public static string enumManifestName
        {
            get { return "NodeControlState"; }
        }

        static NodeControlState()
        {
            EnumManifest.AddEnum(enumManifestName, on, "On");
            EnumManifest.AddEnum(enumManifestName, comply, "Comply");
            EnumManifest.AddEnum(enumManifestName, off, "Off");
            EnumManifest.AddEnum(enumManifestName, parentLink, "Parent Link");
            EnumManifest.AddEnum(enumManifestName, physicsLink, "Physics Link");
            EnumManifest.AddEnum(enumManifestName, hold, "Hold");
            EnumManifest.AddEnum(enumManifestName, lockNode, "Lock");
            EnumManifest.AddEnum(enumManifestName, noChange, "Do not change");

        }
    }
    public static class MultiNodeControlState
    {
        public const int onAllJoints = 1;
        public const int onKeyJoints = 2;
        public const int onKeyElbowKneeJoints = 3;
        public const int complyAllJoints = 4;
        public const int complyKeyJoints = 5;
        public const int complyKeyElbowKneeJoints = 6;
        public const int offAllJoints = 7;
        public const int custom = 10;

        public static string enumManifestName
        {
            get { return "MultiNodeControlState"; }
        }

        static MultiNodeControlState()
        {
            EnumManifest.AddEnum(enumManifestName, onAllJoints, "On: All Joints");
            EnumManifest.AddEnum(enumManifestName, onKeyJoints, "On: Key Joints");
            EnumManifest.AddEnum(enumManifestName, onKeyElbowKneeJoints, "On: Key + Elbow + Knee Joints");
            EnumManifest.AddEnum(enumManifestName, complyAllJoints, "Comply: All Joints");
            EnumManifest.AddEnum(enumManifestName, complyKeyJoints, "Comply: Key Joints");
            EnumManifest.AddEnum(enumManifestName, complyKeyElbowKneeJoints, "Comply: Key + Elbow + Knee Joints");

            EnumManifest.AddEnum(enumManifestName, offAllJoints, "Off: All Joints");

            EnumManifest.AddEnum(enumManifestName, custom, "Custom","Custom...");
        }
    }

    public static class VRControllerDoubleSelectMode
    {
        public const int togglePin = 1;
        public const int toggleHide = 2;
        public const int disabled = 3;

        public static string enumManifestName
        {
            get { return "VRControllerDoubleSelectMode"; }
        }

        static VRControllerDoubleSelectMode()
        {
            EnumManifest.AddEnum(enumManifestName, togglePin, "Toggle Wrist Detach");
            EnumManifest.AddEnum(enumManifestName, toggleHide, "Toggle Hide","Toggle Game Control UI Hide");
            EnumManifest.AddEnum(enumManifestName, disabled, "Disabled", "Disabled");
        }
    }
}
